@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Frank Denis. All rights reserved.
+ * Copyright (c) 2004-2006 Frank Denis. All rights reserved.
  * 
  * This crucial part of UCARP is derived from the OpenBSD project.
  * Original copyright follows. 
@@ -59,7 +59,9 @@ static void carp_set_state(struct carp_softc *sc, int state)
         break;
     case BACKUP:
         logfile(LOG_WARNING, _("Switching to state: BACKUP"));
-        (void) spawn_handler(dev_desc_fd, downscript);
+	if ((sc->sc_state != INIT) || (neutral != 1)) {
+	    (void) spawn_handler(dev_desc_fd, downscript);
+	}
         gratuitous_arp(dev_desc_fd, 0);
         break;
     case MASTER:
@@ -313,17 +315,14 @@ static void carp_setrun(struct carp_softc *sc, sa_family_t af)
 #endif
     switch (sc->sc_state) {
     case INIT:
-        if (preempt != 0 && carp_suppress_preempt == 0) {
-            carp_send_ad(sc);
-            gratuitous_arp(dev_desc_fd, 1);
-#ifdef INET6
-            carp_send_na(sc);
-#endif /* INET6 */
-            carp_set_state(sc, MASTER);
-        } else {
-            carp_set_state(sc, BACKUP);
-            carp_setrun(sc, 0);
-        }
+	if (preempt != 0 && carp_suppress_preempt == 0) {
+	    carp_set_state(sc, MASTER);
+	    carp_send_ad(sc);
+	    gratuitous_arp(dev_desc_fd, 1);
+	} else {
+	    carp_set_state(sc, BACKUP);
+	    carp_setrun(sc, 0);	    
+	}
         break;
     case BACKUP:
         sc->sc_ad_tmo.tv_sec = 0;
@@ -423,10 +422,13 @@ static void packethandler(unsigned char *dummy,
     sp += sizeof etherhead;
     caplen = header->caplen - sizeof etherhead;
     memcpy(&iphead, sp, sizeof iphead);    
+    if (iphead.ip_src.s_addr == srcip.s_addr) {
+	return;
+    }
     ip_len = iphead.ip_hl << 2;
     source = ntohl(iphead.ip_src.s_addr);
     dest = ntohl(iphead.ip_dst.s_addr);
-    proto = iphead.ip_p;    
+    proto = iphead.ip_p;
     switch (proto) {
     case IPPROTO_CARP: {
         struct carp_header ch;
@@ -607,7 +609,9 @@ RETSIGTYPE sighandler_exit(const int sig)
     logfile(LOG_DEBUG, "sighandler_exit(): Calling [%s] and exiting",
             downscript);
 #endif
-    (void) spawn_handler(dev_desc_fd, downscript);
+    if (sc.sc_state != BACKUP) {
+	(void) spawn_handler(dev_desc_fd, downscript);
+    }
     _exit(EXIT_SUCCESS);
 }
 
@@ -687,13 +691,17 @@ int docarp(void)
     pfds[0].fd = dev_desc_fd;
     pfds[0].events = POLLIN | POLLERR | POLLHUP | POLLNVAL;
     
-    carp_setrun(&sc, 0);
-    
     if (shutdown_at_exit != 0) {
         (void) signal(SIGINT, sighandler_exit);
         (void) signal(SIGQUIT, sighandler_exit);
         (void) signal(SIGTERM, sighandler_exit);
     }
+
+    if (gettimeofday(&now, NULL) != 0) {
+	logfile(LOG_WARNING, "initializing now to gettimeofday() failed: %s",
+		strerror(errno));
+    }
+    carp_setrun(&sc, 0); 
     
     for (;;) {
         nfds = poll(pfds, (nfds_t) 1, sc.sc_advbase * 1000);
